#include "WiFi.h"
#include "ESPAsyncWebServer.h"
#include "AsyncTCP.h"
#include "Update.h"

AsyncWebServer server(80);

const char* ssid = "Async";
const char* pass = "1234567890";
unsigned long time1,time2;
int val;

bool shouldReboot = false;

void setup(){
  Serial.begin(115200);
  Serial.println("");
  Serial.println("ESPAsyncWebServer");
  Serial.println("update?");
  //Serial.println("update!");
  Serial.println("");
  WiFi.mode(WIFI_AP);
  WiFi.softAP(ssid, pass);

  server.on("/update", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send(200, "text/html", "<form method='POST' action='/update' enctype='multipart/form-data'><input type='file' name='update'><input type='submit' value='Update'></form>");
  });
  updateOTA();
  server.begin();
}

void loop(){
  if(shouldReboot){
    Serial.println("Rebooting...\n");
    delay(100);
    ESP.restart();
  }
}

void updateOTA() {
  server.on("/update", HTTP_POST, [](AsyncWebServerRequest *request){
    shouldReboot = !Update.hasError();
    AsyncWebServerResponse *response = request->beginResponse(200, "text/plain", shouldReboot?"OK":"FAIL");
    response->addHeader("Connection", "close");
    request->send(response);
    },[](AsyncWebServerRequest *request, String filename, size_t index, uint8_t *data, size_t len, bool final){
    if(!index){
      time1 = millis()/1000;
      Serial.printf("Update Start: %s\n", filename.c_str());
      uint32_t maxSketchSpace = 0x140000;
      Update.runAsync(true);
      if(!Update.begin(maxSketchSpace)){
        Update.printError(Serial);
      }
    }
    if(!Update.hasError()){
      if(Update.write(data, len) != len){
        Update.printError(Serial);
      }
    }
    if(final){
      if(Update.end(true)){
        time2 = millis()/1000; 
        Serial.printf("Update Success: %u B\nUpload time: %d sec.\n", index+len, time2 - time1);
      } else {
        Update.printError(Serial);
      }
    }
  });
}
